﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Configuration;

using Discuz.Common;
using Discuz.Entity;
using Discuz.Forum;
using Discuz.Config;

using Common;
using Common.Web;
using Web.Models;

namespace Web.Controllers
{
    public class APIController : Controller
    {
        #region QQ

        #region 登录

        public ActionResult QQLogin()
        {
            if (-1 == page.UID)
            {
                OAuth.QQ request = new OAuth.QQ();
                string salt;
                string url = request.GetAuthCodeUrl("qqcallback", out salt);
                Session["QQLoginSalt"] = salt;
                return Redirect(url);
            }
            throw new ApplicationException("您已经登录，无须重复登录！若需QQ绑定请点击顶部QQ绑定按钮！");
        }

        public ActionResult QQCallBack()
        {
            if (-1 == page.UID)
            {
                string backSalt = Request.QueryString["state"];
                string code = Request.QueryString["code"];

                if (null != backSalt && null != code && null != Session["QQLoginSalt"] && backSalt == Session["QQLoginSalt"].ToString())
                {
                    Session["QQLoginSalt"] = null;

                    OAuth.QQ request = new OAuth.QQ();

                    string access_token = request.GetAccessToken(code, "callback");

                    string openid = request.GetUID(access_token);

                    int uid = DNT.QQDataAccess.GetUID(openid);

                    if (0 < uid)
                    {
                        return LoginByDzUID(uid);
                    }
                    else
                    {
                        OAuth.QQUser userInfo = request.GetUserInfo(access_token, openid);
                        
                        if (userInfo.Ret == 0 && string.IsNullOrEmpty(userInfo.Msg))
                        {
                            Discuz.Entity.UserInfo user = DNT.DntUser.CreateUser(userInfo.Nickname, "");
                            if (0 < user.Uid)
                            {
                                if (0 < DNT.QQDataAccess.CreateUser(user.Uid, openid))
                                {
                                    return LoginByDzUID(user.Uid);
                                }
                            }
                        }
                        else
                        {
                            throw new ApplicationException(userInfo.Msg);
                        }
                    }
                }
                throw new ApplicationException("验证失败！");
            }
            else
            {
                throw new ApplicationException("您已经登录，无须重复登录！若需QQ绑定请点击顶部QQ绑定按钮！");
            }
        } 

        #endregion

        #region 绑定

        public ActionResult QQBind()
        {
            if (-1 != page.UID)
            {
                string openid = DNT.QQDataAccess.GetOpenID(page.UID.ToString());

                if (string.IsNullOrEmpty(openid))
                {
                    OAuth.QQ request = new OAuth.QQ();
                    string salt;
                    string url = request.GetAuthCodeUrl("qqbindcallback", out salt);
                    Session["QQBindSalt"] = salt;
                    return Redirect(url);
                }
                throw new ApplicationException("您已经绑定过QQ，如需重新绑定，请先用QQ登录解绑，再绑定QQ");
            }
            throw new ApplicationException("若需绑定QQ，请先用其他非QQ帐号登录！");
        }

        public ActionResult QQBindCallBack()
        {
            int uid = page.UID;
            if (-1 != uid)
            {
                string backSalt = Request.QueryString["state"];
                string code = Request.QueryString["code"];

                if (null != backSalt && null != code && null != Session["QQBindSalt"] && backSalt == Session["QQBindSalt"].ToString())
                {
                    Session["QQBindSalt"] = null;

                    OAuth.QQ request = new OAuth.QQ();

                    string access_token = request.GetAccessToken(code, "callback");

                    string openid = request.GetUID(access_token);

                    int openuid = DNT.QQDataAccess.GetUID(openid.ToString());
                    if (0 < openuid)
                    {
                        throw new ApplicationException("您的QQ已绑定过其他用户，如需绑定，请先用QQ登录,解绑后再绑定QQ！");
                    }
                    else
                    {
                        if (0 < DNT.QQDataAccess.CreateUser(uid, openid.ToString()))
                        {
                            return LoginByDzUID(uid);
                        }
                    }
                }
            }
            throw new ApplicationException("若需绑定QQ，请先用其他非QQ帐号登录！");
        } 

        #endregion

        #region 解绑
        public ActionResult QQUnBind()
        {
            if (-1 != page.UID)
            {
                string openid = DNT.QQDataAccess.GetOpenID(page.UID.ToString());

                if (!string.IsNullOrEmpty(openid))
                {
                    DNT.QQDataAccess.DeleteUser(page.UID);

                    Session["uid"] = null;//退出登录

                    return Redirect("/");
                }
                throw new ApplicationException("您还未绑定QQ，无需解绑！");
            }
            throw new ApplicationException("若需解绑，请先登录！");
        } 
        #endregion

        #endregion

        #region Sina

        #region 登录

        public ActionResult SinaLogin()
        {
            if (-1 == page.UID)
            {
                OAuth.Sina request = new OAuth.Sina();
                string salt;
                string url = request.GetAuthCodeUrl("sinacallback", out salt);
                Session["SinaLoginSalt"] = salt;
                return Redirect(url);
            }
            throw new ApplicationException("您已经登录，无须重复登录！若需sina微博绑定请点击顶部sina微博绑定按钮！");
        }

        public ActionResult SinaCallBack()
        {
            if (-1 == page.UID)
            {
                string backSalt = Request.QueryString["state"];
                string code = Request.QueryString["code"];

                if (null != backSalt && null != code && null != Session["SinaLoginSalt"] && backSalt == Session["SinaLoginSalt"].ToString())
                {
                    Session["SinaLoginSalt"] = null;

                    OAuth.Sina request = new OAuth.Sina();

                    OAuth.SinaAccessToken token = request.GetAccessToken(code, "callback");

                    long openid = request.GetUID(token.Access_token);

                    int uid = DNT.SinaDataAccess.GetUID(openid.ToString());

                    if (0 < uid)
                    {
                        return LoginByDzUID(uid);
                    }
                    else
                    {
                        OAuth.SinaUser userInfo = request.GetUserInfo(token.Access_token, openid);

                        Discuz.Entity.UserInfo user = DNT.DntUser.CreateUser(userInfo.screen_name, "");
                        if (0 < user.Uid)
                        {
                            if (0 < DNT.SinaDataAccess.CreateUser(user.Uid, openid.ToString()))
                            {
                                return LoginByDzUID(user.Uid);
                            }
                        }
                    }
                }
                throw new ApplicationException("验证失败！");
            }
            else
            {
                throw new ApplicationException("您已经登录，无须重复登录！若需sina微博绑定请点击顶部sina微博绑定按钮！");
            }
        } 

        #endregion

        #region 绑定

        public ActionResult SinaBind()
        {
            if (-1 != page.UID)
            {
                string openid = DNT.SinaDataAccess.GetOpenID(page.UID.ToString());

                if (string.IsNullOrEmpty(openid))
                {
                    OAuth.Sina request = new OAuth.Sina();
                    string salt;
                    string url = request.GetAuthCodeUrl("sinabindcallback", out salt);
                    Session["SinaBindSalt"] = salt;
                    return Redirect(url);
                }
                throw new ApplicationException("您已经绑定过sina微博，如需重新绑定，请先解绑，再绑定重新sina微博");
            }
            throw new ApplicationException("若需绑定sina微博，请先用其他非sina微博帐号登录！");
        }

        public ActionResult SinaBindCallBack()
        {
            int uid = page.UID;
            if (-1 != uid)
            {
                string backSalt = Request.QueryString["state"];
                string code = Request.QueryString["code"];

                if (null != backSalt && null != code && null != Session["SinaBindSalt"] && backSalt == Session["SinaBindSalt"].ToString())
                {
                    Session["SinaBindSalt"] = null;

                    OAuth.Sina request = new OAuth.Sina();

                    OAuth.SinaAccessToken token = request.GetAccessToken(code, "callback");

                    long openid = request.GetUID(token.Access_token);

                    int openuid = DNT.SinaDataAccess.GetUID(openid.ToString());
                    if (0 < openuid)
                    {
                        throw new ApplicationException("您的sina微博已绑定过其他用户，如需绑定，请先用sina微博登录,解绑后再绑定sina微博！");
                    }
                    else
                    {
                        OAuth.SinaUser userInfo = request.GetUserInfo(token.Access_token, openid);

                        if (0 < DNT.SinaDataAccess.CreateUser(uid, openid.ToString()))
                        {
                            return LoginByDzUID(uid);
                        }
                    }
                }
                throw new ApplicationException("验证失败！");
            }
            throw new ApplicationException("若需绑定sina微博，请先用其他非sina微博帐号登录！");
        } 

        #endregion

        #region 解绑

        public ActionResult SinaUnBind()
        {
            if (-1 != page.UID)
            {
                string openid = DNT.SinaDataAccess.GetOpenID(page.UID.ToString());

                if (!string.IsNullOrEmpty(openid))
                {
                    DNT.SinaDataAccess.DeleteUser(page.UID);

                    Session["uid"] = null;//退出登录

                    return Redirect("/");
                }
                throw new ApplicationException("您还未绑定sina微博，无需解绑！");
            }
            throw new ApplicationException("若需解绑，请先登录！");
        }

        #endregion

        #endregion

        public ActionResult CallBack()
        {
            return new EmptyResult();
        }
        public ActionResult OAuthTop()
        {
            if (0 < page.UID)
            {

                OAuthBindView data = new OAuthBindView();

                data.SinaOpenID = DNT.SinaDataAccess.GetOpenID(page.UID.ToString());
                data.QQOpenID = DNT.QQDataAccess.GetOpenID(page.UID.ToString());

                return PartialView("~/Views/Shared/OAuthTop.ascx", data);
            }

            else throw new ApplicationException("请先登录");
        }

        private ActionResult LoginByDzUID(int uid)
        {
            GeneralConfigInfo config = GeneralConfigs.GetConfig();
            ShortUserInfo userInfo = Discuz.Forum.Users.GetShortUserInfo(uid);
            ForumUtils.WriteUserCookie(userInfo.Uid, TypeConverter.StrToInt(DNTRequest.GetString("expires"), -1),
                config.Passwordkey, DNTRequest.GetInt("templateid", 0), DNTRequest.GetInt("loginmode", -1));

            Discuz.Forum.Users.UpdateUserCreditsAndVisit(userInfo.Uid, DNTRequest.GetIP());

            Session["uid"] = uid;//站点其他登录信息

            return Redirect("/");
        }

        DNT.DntPage page = new DNT.DntPage();
    }
}
